#!/usr/bin/python
# coding=utf-8
import click
from .init import generate_project


def _validate_project_name(ctx, argument, value):
    """
    验证输入的项目名称是否正确
    """
    if not value or ' ' in value:
        raise click.BadParameter(u"项目名称中不能含有空格")
    return value


def _validate_project_type(ctx, argument, value):
    """
    验证输入的项目类型是否正确
    """
    if not value in ('app', 'api'):
        raise click.BadParameter(u"项目类型不正确")
    return value


@click.group()
@click.version_option()
def cli():
    """
     pithy-cli是一个接口测试项目生成脚手架,功能完善中
    """


@cli.command()
@click.pass_context
@click.option('--project-type', prompt=click.style(u"请选择项目类型,输入api或者app", fg='green'),
              callback=_validate_project_type, help=u"请选择项目类型,输入api或者app")
@click.option('--project-name', prompt=click.style(u"请输入项目名称,如pithy-api-test", fg='green'),
              callback=_validate_project_name, help=u"请输入项目名称,如pithy-api-test")
def init(ctx, project_type, project_name):
    """
    生成接口测试项目

    使用方法:
        $ pithy-cli init         # 生成接口测试项目
    """
    generate_project(project_name, project_type)



#!/usr/bin/python
# coding=utf-8
from .api import request, make_session, response
# from .thrift_client import thrift_client
# from .db import DB as db
from .human_time import HumanDateTime
from .json_processor import JSONProcessor
from .cfg import Config
from .utils import pretty_print

get_config = config_manager = Config



# coding=utf-8
import time
import os
from selenium import webdriver
from selenium.webdriver.remote.webdriver import WebElement
from selenium.webdriver.common.by import By

UPLOAD_EXE_PATH = os.path.join(os.path.dirname(__file__), 'upload.exe')


def time_check(func):
    def wrapper(*args, **kwargs):
        t1 = time.time()
        res = func(*args, **kwargs)
        t2 = time.time()
        print(u'查找【%s】元素用时%.3f秒' % (args[1], t2 - t1))
        return res
    return wrapper


class BasePage(object):
    driver = None  # type:webdriver.Chrome

    def __new__(cls, *args, **kwargs):
        if not cls.driver:
            cls.driver = webdriver.Chrome()
        return object.__new__(cls, *args, **kwargs)

    def element(self, item):
        """
        :rtype: WebElement
        """
        try:
            value = self.locators[item]
        except:
            raise Exception(u'没有定义【%s】元素的定位方式，请检查locators' % item)
        return self.find_element(item, value)

    def elements(self, item):
        """
        :rtype: list of WebElement
        """
        try:
            value = self.locators[item]
        except:
            raise Exception(u'没有定义【%s】元素的定位方式，请检查locators' % item)
        return self.find_elements(item, value)

    @staticmethod
    def upload(file_path):
        os.system('{exe_path} {file_path}'.format(exe_path=UPLOAD_EXE_PATH, file_path=file_path))

    @time_check
    def find_element(self, item, value):
        try:
            return self.driver.find_element(*value)
        except:
            raise Exception(u'没有找到元素【%s】' % item)

    @time_check
    def find_elements(self, item, value):
        try:
            return self.driver.find_elements(*value)
        except:
            raise Exception(u'没有找到元素【%s】' % item)

    @classmethod
    def quit(cls):
        try:
            cls.driver.quit()
        except:
            pass
        finally:
            cls.driver = None



#!/usr/bin/python
# coding=utf-8
from pithy import request
import unittest


class DemoAPP(object):

    def __init__(self):
        self.base_url = 'http://httpbin.org/'

    @request(url='get')
    def get(self, value):
        params = {
            'key': value
        }
        return dict(params=params)


class TestApi(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.app = DemoAPP()

    def test_get(self):
        res = self.app.get(123).json
        assert res.args.key == '123'



#!/usr/bin/python
# coding=utf-8
import os
import sys
import contextlib
from jinja2 import FileSystemLoader, Environment
from click import echo, style

template_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates')

jinja_env = (lambda: Environment(loader=FileSystemLoader(template_dir)))()


@contextlib.contextmanager
def cd(d):
    cwd = os.getcwd()
    os.chdir(d)
    yield
    os.chdir(cwd)


def mkdir_project(project_name):
    if os.path.exists(project_name) and os.path.isdir(project_name):
        return
    try:
        os.mkdir(project_name)
    except OSError as e:
        echo(style(u"创建 %s 失败, %s" % (project_name, str(e)), fg='red'))
        sys.exit(1)


def get_project_path(project_name):
    return os.path.join(os.getcwd(), project_name)


def get_templates(project_type):
    templates = []

    def get_file_name(folder, base_dir):
        for name in os.listdir(base_dir):
            full_path = os.path.join(base_dir, name)
            if os.path.isfile(full_path):
                templates.append(os.path.join(project_type, folder, name))
            else:
                get_file_name(name, full_path)

    project_template_dir = os.path.join(template_dir, project_type)
    get_file_name('', project_template_dir)
    templates = [file_name.replace('\\', '/') for file_name in templates]
    return templates


def render_template(jinja_env, template, target_full_name, **kwargs):
    """
    渲染横版
    :param jinja_env: jinja环境对象
    :param template: 模版名称
    :param target_full_name: 目录文件全路径
    :param kwargs: 参数
    :return:
    """

    content = jinja_env.get_template(template).render(**kwargs).encode('utf-8')
    target_dir = os.path.dirname(target_full_name)
    if not os.path.exists(target_dir):
        os.makedirs(target_dir)

    with open(target_full_name, 'wb') as f:
        f.write(content)



# coding=utf-8

import unittest
from pithy import Config


class TestConfig(unittest.TestCase):

    def test_json_config(self):
        cfg = Config('cfg.json')
        assert cfg['a'] == 1
        assert cfg['b'] == [2, 3]
        assert cfg['c'] == {'d': 1}



# coding=utf-8
import json
import time
import sys
from functools import wraps

python_version = sys.version[0]

if python_version == '3':
    basestring = str


def fn_timer(function):
    """
    元素查找计时器
    """
    @wraps(function)
    def function_timer(*args, **kwargs):
        t0 = time.time()
        result = function(*args, **kwargs)
        t1 = time.time()
        total_time = str(t1 - t0)
        return total_time, result

    return function_timer


def format_json(content):
    """
    格式化JSON
    """
    if isinstance(content, basestring):
        content = json.loads(content)

    if python_version == '3':
        result = json.dumps(content, sort_keys=True, indent=4, separators=(',', ': ')). \
            encode('latin-1').decode('unicode_escape')
    else:
        result = json.dumps(content, sort_keys=True, indent=4, separators=(',', ': ')). \
            decode("unicode_escape")

    return result


def pretty_print(content):
    """
    美化打印
    """
    print(format_json(content))



#!/usr/bin/python
# coding=utf-8
import os
import sys
import json
import yaml
from configobj import ConfigObj


class Config(object):
    """
    读取配置文件
    注意:
      1、优先读取当前目录下, 第二顺序读取项目根目录下
      2、同一名称配置文件只加载一次,再次实例化不再加载

    使用:
    from pithy import Config
    config = Config()  # 使用默认配置文件名cfg.yaml
    config = Config('pithy.yaml')  # 使用自定义配置文件

    print config['pithy_db']['host']  # 取值
    ...

    """
    config_object_instance = {}

    def __new__(cls, file_name='cfg.yaml'):
        if file_name not in cls.config_object_instance:
            config_file_path = file_name
            if not os.path.exists(file_name):
                config_file_path0 = os.path.join(sys.path[0], file_name)
                config_file_path1 = os.path.join(sys.path[1], file_name)
                if os.path.exists(config_file_path0):
                    config_file_path = config_file_path0
                elif os.path.exists(config_file_path1):
                    config_file_path = config_file_path1
                else:
                    raise OSError('can not find config file !')
            if file_name.endswith('.yaml'):
                cls.config_object_instance[file_name] = yaml.load(open(config_file_path))
            elif file_name.endswith(('.cfg', '.ini', '.conf')):
                cls.config_object_instance[file_name] = ConfigObj(config_file_path)
            elif file_name.endswith('.json'):
                cls.config_object_instance[file_name] = json.load(open(config_file_path))
            else:
                raise ValueError('Unsupported configuration file type')

        return cls.config_object_instance[file_name]

    def __getitem__(self, item):
        return self[item]



#!/usr/bin/python
# coding=utf-8
import os
import sys
import thriftpy
from thriftpy.rpc import make_client

LOG_FORMAT = u'''
********************************************

1、请求方法:
{thrift_module_name}.{service_name}.{function_name}

2、请求参数:
{thrift_kwargs}

3、响应结果:
{response}

'''


def thrift_service_wrapper(func):
    def get_params(func, *args, **kwargs):
        get_params.locals = {}

        def tracer(frame, event, arg):
            if event == 'return':
                get_params.locals = frame.f_locals.copy()

        sys.setprofile(tracer)
        func(*args, **kwargs)
        params = get_params.locals
        get_params.locals = {}
        return params

    def assemble_struct(struct_obj, dict_obj):
        for value in struct_obj.thrift_spec.values():
            if isinstance(value[2], thriftpy.thrift.TPayloadMeta):
                dict_obj[value[1]] = assemble_struct(value[2], dict_obj[value[1]])
            elif isinstance(value[2], (tuple, list)):
                if isinstance(value[2][1], thriftpy.thrift.TPayloadMeta):
                    dict_obj[value[1]] = [assemble_struct(value[2][1], d) for d in dict_obj[value[1]]]
        return struct_obj(**dict_obj)

    def assemble_kwargs(function_args_dict, params_dict):
        result = {}
        for value in function_args_dict.thrift_spec.values():
            if isinstance(value[2], thriftpy.thrift.TPayloadMeta):
                result[value[1]] = assemble_struct(value[2], params_dict[value[1]])
            elif isinstance(value[2], (tuple, list)):
                if isinstance(value[2][1], thriftpy.thrift.TPayloadMeta):
                    result[value[1]] = [assemble_struct(value[2][1], d) for d in params_dict[value[1]]]
            else:
                result[value[1]] = params_dict[value[1]]

        return result

    def function_wrapper(*args, **kwargs):
        function_name = func.__name__
        service_class_ins = func.im_self
        service_name = service_class_ins.__class__.__name__
        thrift_module = service_class_ins.thrift_module
        thrift_service_ins = getattr(thrift_module, service_name)
        function_args_dict = getattr(thrift_service_ins, function_name + '_args')
        user_params_dict = get_params(func, *args, **kwargs)
        thrift_kwargs = assemble_kwargs(function_args_dict, user_params_dict)
        client = make_client(thrift_service_ins, host=service_class_ins.host, port=int(service_class_ins.port))

        try:
            response = getattr(client, function_name)(**thrift_kwargs)
        finally:
            client.close()

        print(LOG_FORMAT.format(thrift_module_name=thrift_module.__name__,
                                service_name=service_name,
                                function_name=function_name,
                                thrift_kwargs=thrift_kwargs,
                                response=response))
        return response
    return function_wrapper


def thrift_client(cls):
    def cls_wrapper(*args, **kwargs):
        def __getattribute__(self, item):
            thrift_module = object.__getattribute__(self, 'thrift_module')
            service_name = object.__getattribute__(self, '__class__').__name__
            thrift_services = getattr(thrift_module, service_name)
            thrift_services_list = thrift_services.thrift_services
            if item in thrift_services_list:
                return thrift_service_wrapper(object.__getattribute__(self, item))
            else:
                return object.__getattribute__(self, item)

        service_client_instance = cls(*args, **kwargs)
        thrift_file = service_client_instance.thrift_file
        thrift_module_name = os.path.splitext(os.path.basename(thrift_file))[0] + '_thrift'
        thrift_module = thriftpy.load(thrift_file, module_name=thrift_module_name)
        setattr(cls, 'thrift_module', thrift_module)
        setattr(cls, '__getattribute__', __getattribute__)
        return service_client_instance

    return cls_wrapper



#!/usr/bin/python
# coding=utf-8
import os
from click import echo, style
from .utils import mkdir_project, get_project_path, get_templates, cd, render_template, jinja_env


def generate_project(project_name, project_type):
    """
    生成项目
    :param project_name: 项目名称
    :param project_type: 项目类型
    """
    echo(style(u'开始创建%s项目' % project_name, fg='red'))

    # 创建项目文件夹
    mkdir_project(project_name)
    project_path = get_project_path(project_name)

    # 获取所有模版文件
    templates = get_templates(project_type)

    with cd(project_path):
        echo(style(u'开始渲染...', fg='green'))
        for template in templates:
            target_file_name = os.path.splitext(template)[0]
            target_full_name = os.path.join(project_path, target_file_name[len(project_type) + 1:])
            try:
                render_template(jinja_env, template, target_full_name, project_name=project_name)
                echo(style(u'生成 %-32s ' % target_file_name, fg='green') +
                     style(u'[√]', fg='blue'))
            except Exception as e:
                echo(style(u'生成 %-32s ' % target_file_name, fg='red') +
                     style(u'[×]\n', fg='blue'))
                raise Exception
    echo(style(u'生成成功,请使用编辑器打开该项目', fg='red'))


if __name__ == '__main__':
    generate_project('abc', 'interface')


#!/usr/bin/python
# coding=utf-8
from setuptools import setup, find_packages

setup(
    name="pithy-test",
    version="0.0.17",
    keywords=("interface", "automation", "testing", "pithy"),
    description=u"简化接口测试",
    long_description=u"简化接口测试",
    url="https://github.com/yuyu1987/pithy-test",
    author="coolfish",
    author_email="hgbaczt@gmail.com",
    packages=['pithy'],
    include_package_data=True,
    platforms="any",
    install_requires=['requests', 'records', 'objectpath', 'click',
                      'jinja2', 'pyyaml', 'configobj'],
    entry_points='''
    [console_scripts]
    pithy-cli=pithy.cli:cli
    '''
)



#!/usr/bin/python
# coding=utf-8
import sys
from sqlalchemy.ext.automap import automap_base
from sqlalchemy.orm import Session
from sqlalchemy import create_engine


class _DB(object):
    def __init__(self, database_info, name):
        self.database_info = database_info
        self.database_name = name
        self.base = None
        self.session = None

    def load(self):
        self._init_db()
        sys.modules[self.database_name] = self
        return self

    def _init_db(self):
        self.base = automap_base()
        engine = create_engine(self.database_info)
        self.base.prepare(engine, reflect=True)
        self.session = Session(engine)

    def __getattr__(self, table_name):
        try:
            table_object = getattr(self.base.classes, table_name)
        except AttributeError:
            raise Exception(u'没找到:{table_name}'.format(table_name=table_name))
        return table_object


class DB(object):
    """
    example:
    DB("mysql://user:password@host:port/database?charset=utf8", 'pithy_db')
    from pithy_db import o_order, session
    query_result = session.query(o_order).all()
    """

    db_pool = {}

    def __new__(cls, database_info, name):
        if name in cls.db_pool:
            return cls.db_pool[name]
        else:
            db_module = _DB(database_info, name).load()
            cls.db_pool[name] = db_module
            return db_module



#!/usr/bin/python
# coding=utf-8
import logging
from requests.sessions import Session
from functools import wraps
from jinja2 import Template
from copy import deepcopy
from collections import OrderedDict
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin
import inspect

from .utils import format_json
from .json_processor import JSONProcessor


class HttpRequest(object):
    def __init__(self, url='', method='get', **kwargs):
        self.url = url
        self.method = method
        self.decorator_args = kwargs
        self.func_return = None
        self.func_doc = None
        self.func_im_self = None
        self.session = None

    def __call__(self, func):
        self.func = func
        self.is_class = False
        try:
            if inspect.getfullargspec(func).args[0] == 'self':
                self.is_class = True
        except IndexError:
            pass

        def fun_wrapper(*args, **kwargs):
            self.func_return = self.func(*args, **kwargs) or {}
            self.func_im_self = args[0] if self.is_class else object

            try:
                self.func.__doc__ = self.func.__doc__.decode('utf-8')
            except:
                pass
            self.func_doc = (self.func.__doc__ or self.func.__name__).strip()
            self.create_url()
            self.create_session()
            self.session.headers.update(getattr(self.func_im_self, 'headers', {}))
            self.decorator_args.update(self.func_return)
            return Request(self.method, self.url, self.session, self.func_doc, self.decorator_args)
        return fun_wrapper

    def create_url(self):
        """
        生成http请求的url
        """

        # 使用在函数中定义的url变量,如果没有,使用装饰器中定义的
        base_url = getattr(self.func_im_self, 'base_url', '')
        self.url = self.func_return.pop('url', None) or self.url
        self.url = urljoin(base_url, self.url)

    def create_session(self):
        """
        如果接收到的要变参数中有session,且为Session对象,赋值给session变量, 否则创建一个
        """
        if self.is_class:
            self.session = getattr(self.func_im_self, 'session', None)
            if not isinstance(self.session, Session):
                session = Session()
                setattr(self.func_im_self, 'session', session)
                self.session = session

        elif isinstance(self.func_return.get('session'), Session):
            self.session = self.func_return.get('session')
        else:
            self.session = Session()


request = HttpRequest

LOG_TEMPLATE = u'''
******************************************************
{% for index, item in items %}
{{ index + 1 }}、{{ item['desc'] }}
{{ item['value'] }}
{% endfor %}
'''


def context(func):
    def wrapper(self):
        self._request()
        try:
            res = func(self)
        finally:
            self._log()
        return res
    return wrapper


class Request(object):
    """
    请求对象模型
    """

    def __init__(self, method, url, session, doc, args):
        self.method = method
        self.url = url
        self.session = session
        self.doc = doc
        self.args = args
        self.response = None
        self.log_content = [
            dict(desc=u'接口描述', value=doc),
            dict(desc=u'请求url', value=url),
            dict(desc=u'请求方法', value=method),
        ]

        for i in ['params', 'data']:
            if args.get(i):
                args[i] = self.fixation_order(args[i])

    def get_arg(self, arg):
        return self.args.get(arg)

    def add_headers(self, **kwargs):
        if self.args.get('headers'):
            self.args['headers'].update(kwargs)
        else:
            self.args['headers'] = kwargs

    @staticmethod
    def fixation_order(d):
        o = OrderedDict()
        for i in d:
            o[i] = d[i]
        return o

    def prepare_log(self):
        headers = deepcopy(self.args.get('headers', {}))
        headers.update(self.session.headers)

        if headers:
            self.log_content.append(dict(
                desc=u'请求headers', value=format_json(headers)
            ))

        if self.args.get('params'):
            self.log_content.append(dict(
                desc=u'请求url参数', value=format_json(self.args.get('params'))
            ))

        if self.args.get('data'):
            self.log_content.append(dict(
                desc=u'body参数', value=format_json(self.args.get('data'))
            ))

        if self.args.get('json'):
            self.log_content.append(dict(
                desc=u'body参数', value=format_json(self.args.get('json'))
            ))

    @context
    def to_json(self):
        try:
            response_json = self.response.json()
            self.log_content.append(dict(
                desc=u'响应结果',
                value=format_json(response_json)
            ))
        except ValueError:
            self.log_content.append(dict(
                desc=u'响应结果',
                value=self.response.content.decode('utf-8')
            ))
            raise ValueError(u'No JSON object in response')

        return JSONProcessor(response_json)

    @context
    def to_content(self):
        response_content = self.response.content
        self.log_content.append(dict(desc=u'响应结果', value=response_content.decode('utf-8')))
        return response_content

    @property
    def json(self):
        return self.to_json()

    @property
    def content(self):
        return self.to_content()

    def _log(self):
        print(Template(LOG_TEMPLATE).render(items=enumerate(self.log_content)))

    def _request(self):
        if not self.response:
            self.prepare_log()
            self.response = self.session.request(self.method, self.url, **self.args)

    def __getattr__(self, item):
        self._request()
        self._log()
        return getattr(self.response, item)


def make_session():
    return Session()


class response(Request):
    def __new__(cls, **kwargs):
        """
        :rtype: Request
        """
        return dict(**kwargs)



#!/usr/bin/python
# coding=utf-8
from datetime import datetime, timedelta
import time

# 日期格式
datetime_formats = ["%Y-%m-%d %H:%M:%SZ",
                    "%Y-%m-%d %H:%M:%S",
                    "%Y-%m-%d %H:%M:%S%z",
                    "%Y-%m-%d %H:%M:%S.%f",
                    "%Y-%m-%d %H:%M:%S %f",
                    "%Y-%m-%d %H:%M:%S%Z",
                    "%c",
                    "%s",
                    "%Y-%m-%d",
                    # YMD other than ISO
                    "%Y%m%d",
                    "%Y.%m.%d",
                    # Popular MDY formats
                    "%m/%d/%Y",
                    "%m/%d/%y",
                    # DMY with full year
                    "%d %m %Y",
                    "%d-%m-%Y",
                    "%d/%m/%Y",
                    "%d/%m %Y",
                    "%d.%m.%Y",
                    "%d. %m. %Y",
                    "%d %b %Y",
                    "%d %B %Y",
                    "%d. %b %Y",
                    "%d. %B %Y",
                    # MDY with full year
                    "%b %d %Y",
                    "%b %dst %Y",
                    "%b %dnd %Y",
                    "%b %drd %Y",
                    "%b %dth %Y",
                    "%b %d, %Y",
                    "%b %dst, %Y",
                    "%b %dnd, %Y",
                    "%b %drd, %Y",
                    "%b %dth, %Y",
                    "%B %d %Y",
                    "%B %dst %Y",
                    "%B %dnd %Y",
                    "%B %drd %Y",
                    "%B %dth %Y",
                    "%B %d, %Y",
                    "%B %dst, %Y",
                    "%B %dnd, %Y",
                    "%B %drd, %Y",
                    "%B %dth, %Y",
                    # DMY with 2-digit year
                    "%d %m %y",
                    "%d-%m-%y",
                    "%d/%m/%y",
                    "%d/%m-%y",
                    "%d.%m.%y",
                    "%d. %m. %y",
                    "%d %b %y",
                    "%d %B %y",
                    "%d. %b %y",
                    "%d. %B %y",
                    # MDY with 2-digit year
                    "%b %dst %y",
                    "%b %dnd %y",
                    "%b %drd %y",
                    "%b %dth %y",
                    "%B %dst %y",
                    "%B %dnd %y",
                    "%B %drd %y",
                    "%B %dth %y"
                    ]


class HumanDateTime(object):
    """
    # 解析时间戳
    print(repr(HumanDateTime(1490842267)))
    print(HumanDateTime(1490842267000))
    print(HumanDateTime(1490842267.11111))
    print(HumanDateTime(1490842267111.01))

    # 解析字符串格式日期
    print(HumanDateTime('2017-02-02'))
    print(HumanDateTime('Thu Mar 30 14:21:20 2017'))
    print(HumanDateTime(time.ctime()))
    print(HumanDateTime('2017-3-3'))
    print(HumanDateTime('3/3/2016'))
    print(HumanDateTime('2017-02-02 00:00:00'))

    # 解析datetime或date类型时间
    print(HumanDateTime(datetime(year=2018, month=11, day=30, hour=11)))
    print(HumanDateTime(date(year=2018, month=11, day=30)))

    # 增加减少时间
    print(HumanDateTime('2017-02-02').add_day(1))
    print(HumanDateTime('2017-02-02').sub_day(1))
    print(HumanDateTime('2017-02-02').add_hour(1))
    print(HumanDateTime('2017-02-02').sub_hour(1))
    print(HumanDateTime('2017-02-02').add(days=1, hours=1, weeks=1, minutes=1, seconds=6))
    print(HumanDateTime('2017-02-02').sub(days=1, hours=1, weeks=1, minutes=1, seconds=6))

    # 转换为时间戳
    print(HumanDateTime(1490842267.11111).timestamp_second)
    print(HumanDateTime(1490842267.11111).timestamp_microsecond)
    print(HumanDateTime('2017-02-02 12:12:12.1111').add_day(1).timestamp_microsecond)
    print(HumanDateTime('2017-02-02 12:12:12 1111').add_day(1).timestamp_microsecond)

    # 比较大小
    print(HumanDateTime('2017-02-02 12:12:12 1111') < HumanDateTime('2017-02-02 12:12:11 1111'))
    print(HumanDateTime('2017-02-02 12:12:12 1111') < HumanDateTime('2017-02-02 12:13:11 1111'))
    print(HumanDateTime('2017-02-02 12:12:12 1111') < '2017-02-02 12:11:11')
    print(HumanDateTime('2017-02-02 12:12:12 1111') < '2017-02-02 12:13:11 1111')
    print(HumanDateTime('2017-02-02 12:12:12 1111') == '2017-02-02 12:13:11 1111')
    print(HumanDateTime('2017-02-02 12:12:12 1111') == '2017-02-02 12:13:12 1111')
    print(HumanDateTime('2017-02-02 12:12:12 1111') <= '2017-02-02 12:13:11 1111')
    print(HumanDateTime('2017-02-02 12:12:12 1111') >= '2017-02-02 12:13:11 1111')
    print(HumanDateTime('2017-02-02 12:12:12 1111') != time.time())
    print(HumanDateTime('2017-02-02 12:12:12 1111') <= time.time())
    print(HumanDateTime('2017-02-02 12:12:12 1111') >= time.time())

    # 约等于或者接近
    print(HumanDateTime('2017-02-02 12:12:12 1111').approach('2017-02-02 12:12:11 1111'))
    print(HumanDateTime('2017-02-02 12:12:12 1111').approach('2017-02-02 12:12:10 1111'))
    print(HumanDateTime('2017-02-02 12:12:12 1111').approach('2017-02-02 12:12:10 1111', offset=2))
    print(HumanDateTime('2017-02-02 12:12:12 1111').approach('2017-02-02 12:12:14 1111', offset=2))

    # 调用datetime的方法和属性
    print(HumanDateTime('2017-02-02 12:12:12 1111').day)
    print(HumanDateTime('2017-02-02 12:12:12 1111').year)
    print(HumanDateTime('2017-02-02 12:12:12 1111').second)
    print(HumanDateTime('2017-02-02 12:12:12 1111').date())
    """
    def __init__(self, time_object=None):

        self._datetime = None

        # 如果没有初始值的话,time_object设置为当前时间
        if not time_object:
            time_object = datetime.now()

        time_object = str(time_object)

        # 如果time_object可转化为数字,则使用时间戳解析,如果不能,使用字符串方式解析
        try:
            time_object = float(time_object)
            self._parse_timestamp(time_object)
        except ValueError:
            self._parse_time_str(time_object)

    def _parse_timestamp(self, time_object):
        """
        解析时间戳
        """
        if len(str(int(time_object))) == 10:
            self._datetime = datetime.fromtimestamp(time_object)
        elif len(str(int(time_object))) == 13:
            self._datetime = datetime.fromtimestamp(time_object / 1000.0)
        else:
            raise ValueError(u'输入的时间格式不正确')

    def _parse_time_str(self, time_object):
        """
        解析时间字符串
        """
        for i in datetime_formats:
            try:
                self._datetime = datetime.strptime(time_object, i)
                break
            except:
                continue

        if not self._datetime:
            raise ValueError(u'输入的时间格式不正确')

    @property
    def timestamp_second(self):
        """
        时间戳(秒)
        """
        return int(self._datetime.strftime("%s"))

    @property
    def timestamp_microsecond(self):
        """
        时间戳(毫秒)
        """
        return int(time.mktime(self._datetime.timetuple()) * 1e3 + self._datetime.microsecond / 1e3)

    def add(self, days=0, weeks=0, hours=0, minutes=0, seconds=0):
        """
        增加时间
        :param days: 天
        :param weeks: 星期
        :param hours: 小时
        :param minutes: 分钟
        :param seconds: 秒
        :return:
        """
        return self.__class__(str(self._datetime +
                                  timedelta(days=days, weeks=weeks, hours=hours, minutes=minutes, seconds=seconds)))

    def sub(self, days=0, weeks=0, hours=0, minutes=0, seconds=0):
        """
        减少时间
        :param days: 天
        :param weeks: 星期
        :param hours: 小时
        :param minutes: 分钟
        :param seconds: 秒
        :return: HumanDateTime对象
        """
        return self.__class__(str(self._datetime +
                                  timedelta(days=-days, weeks=-weeks, hours=-hours, minutes=-minutes, seconds=-seconds)))

    def add_week(self, week):
        """
        增加星期
        :param week: 星期
        :return: HumanDateTime对象
        """
        return self.add(weeks=week)

    def sub_week(self, week):
        """
        减少星期
        :param week: 星期
        :return: HumanDateTime对象
        """
        return self.sub(weeks=week)

    def add_day(self, day):
        """
        增加天
        :param day: 天
        :return: HumanDateTime对象
        """
        return self.add(days=day)

    def sub_day(self, day):
        """
        减少天
        :param day: 天
        :return: HumanDateTime对象
        """
        return self.sub(days=day)

    def add_hour(self, hour):
        """
        增加小时
        :param hour: 小时
        :return: HumanDateTime对象
        """
        return self.add(hours=hour)

    def sub_hour(self, hour):
        """
        减少小时
        :param hour: 小时
        :return: HumanDateTime对象
        """
        return self.sub(hours=hour)

    def add_minute(self, minute):
        """
        增加分钟
        :param minute: 分钟
        :return: HumanDateTime对象
        """
        return self.add(minutes=minute)

    def sub_minute(self, minute):
        """
        减少分钟
        :param minute: 分钟
        :return: HumanDateTime对象
        """
        return self.sub(minutes=minute)

    def approach(self, other, offset=1):
        """
        约等于（接近）
        :param other: 比较对象
        :param offset: 允许的偏差,单位为秒
        :return:
        """
        return abs(self.timestamp_microsecond - self.__class__(other).timestamp_microsecond) <= offset * 1000

    def __lt__(self, other):
        return self.timestamp_microsecond < self.__class__(other).timestamp_microsecond

    def __le__(self, other):
        return self.timestamp_microsecond <= self.__class__(other).timestamp_microsecond

    def __gt__(self, other):
        return self.timestamp_microsecond > self.__class__(other).timestamp_microsecond

    def __ge__(self, other):
        return self.timestamp_microsecond >= self.__class__(other).timestamp_microsecond

    def __eq__(self, other):
        return self.timestamp_microsecond == self.__class__(other).timestamp_microsecond

    def __ne__(self, other):
        return self.timestamp_microsecond != self.__class__(other).timestamp_microsecond

    def __getattr__(self, item):
        return getattr(self._datetime, item)

    def __str__(self):
        return str(self._datetime)

    def __unicode__(self):
        return unicode(self._datetime)

    def __repr__(self):
        return repr(self._datetime)



#!/usr/bin/python
# coding=utf-8
from pithy import JSONProcessor
import json
import unittest


class TestJSONProcessor(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.raw_dict = {
            'key1': 1,
            'key2': [1, 2, 3, 4],
            'key3': {
                'key31': 1,
                'key32': '我'
            }
        }

        cls.raw_json = json.dumps(cls.raw_dict)

    def test_1(self):
        jp = JSONProcessor(self.raw_dict)
        assert jp.key1 == 1
        assert jp.key3.key31 == 1
        assert jp.key3.key32 == '我'

        jp2 = JSONProcessor(self.raw_json)
        assert jp2.key1 == 1
        assert jp2.key3.key31 == 1
        assert jp2.key3.key32 == u'我'

    def test_2(self):
        jp = JSONProcessor(self.raw_dict, key4=4)
        assert jp.key4 == 4

        jp2 = JSONProcessor(self.raw_json, {'key4': 4})
        assert jp2.key4 == 4

    def test_3(self):
        jp = JSONProcessor(self.raw_dict)
        assert jp('$.key1') == 1

    def test_4(self):
        jp = JSONProcessor(self.raw_dict)
        print(str(jp))
        print(unicode(jp))
